/* KaspRootRepository.hh */
/* Created by Enomoto Sanshiro on 26 June 2002. */
/* Last updated by Enomoto Sanshiro on 26 June 2002. */


#ifndef __KaspRootRepository_hh__
#define __KaspRootRepository_hh__


#include <string>
#include <fstream>
#include "KaspConfig.hh"
#include "KaspRepository.hh"


#if IS_ROOT_AVAILABLE

class TFile;

class TKaspRootRepository: public TKaspRepository {
  public:
    TKaspRootRepository(const std::string& FileName);
    virtual ~TKaspRootRepository();
    virtual void SaveNtuple(const std::string& Name, TKaspNtuple* Ntuple) throw(TKaspException);
    virtual void SaveGraph(const std::string& Name, TKaspGraph* Graph) throw(TKaspException);
    virtual void SaveHistogram(const std::string& Name, TKaspHistogram* Histogram) throw(TKaspException);
    virtual void Save2dHistogram(const std::string& Name, TKasp2dHistogram* Histogram) throw(TKaspException);
    virtual void SaveTrend(const std::string& Name, TKaspTrend* Trend) throw(TKaspException);
    virtual void SaveHistory(const std::string& Name, TKaspHistory* History) throw(TKaspException);
    virtual void SaveWave(const std::string& Name, TKaspWave* Wave) throw(TKaspException);
    virtual void SaveMap(const std::string& Name, TKaspMap* Map) throw(TKaspException);
    virtual void SaveTabular(const std::string& Name, TKaspTabular* Tabular) throw(TKaspException);
    virtual TKaspNtuple* LoadNtuple(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspGraph* LoadGraph(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspHistogram* LoadHistogram(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKasp2dHistogram* Load2dHistogram(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspTrend* LoadTrend(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspHistory* LoadHistory(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspWave* LoadWave(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspMap* LoadMap(const std::string& Name, int Revision = 0) throw(TKaspException);
    virtual TKaspTabular* LoadTabular(const std::string& Name, int Revision = 0) throw(TKaspException);
  protected:
    virtual void OpenToWrite(void) throw(TKaspException);
    virtual void OpenToRead(void) throw(TKaspException);
    virtual void Close(void);
  protected:
    std::string _FileName;
    TFile* _InputFile;
    TFile* _OutputFile;
};

#endif


#endif
